/*
 *
 *	Common things for all utilities
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include <sys/types.h>

#include "common.h"

void die(int ret, char *fmtstr, ...)
{
	va_list args;

	va_start(args, fmtstr);
	vfprintf(stderr, fmtstr, args);
	va_end(args);
	exit(ret);
}

void *smalloc(size_t size)
{
	void *ret = malloc(size);
	if (!ret) {
		puts("Not enough memory.\n");
		exit(3);
	}
	return ret;
}

void sstrncpy(char *d, char *s, int len)
{
	strncpy(d, s, len);
	d[len-1] = 0;
}

void sstrncat(char *d, char *s, int len)
{
	strncat(d, s, len);
	d[len-1] = 0;
}

void version(void)
{
	printf("Quota utilities version %s (kernel versions %s) from %s.\n", UTIL_VERSION, KERN_VERSION, UTIL_DATE);
#if defined(RPC) || defined(EXT2_DIRECT)
	printf("Compiled with ");
#if defined(RPC) && defined(EXT2_DIRECT)
	puts("RPC and EXT2_DIRECT");
#elif defined(RPC)
	puts("RPC");
#else
	puts("EXT2_DIRECT");
#endif	/* defined RPC && EXT2_DIRECT */
#endif	/* defined RPC || EXT2_DIRECT */
}

/* Compare two paths */
int pathcmp(const char *s1, const char *s2)
{
	while (*s1 && *s2 && *s1++ == *s2++);
	if (!*s2 && !*s2)
		return 0;
	if ((!*s1 && !strcmp(s2, "/")) || (!*s2 && !strcmp(s1, "/")))	/* Trailing slash? */
		return 0;
	return 1;
}
