/*
 *
 *	Utility for turning quotas on and off
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <linux/quota.h>

#include "common.h"
#include "quotasys.h"

#define FL_VERBOSE 1
#define FL_ALL 2
#define FL_USER 4
#define FL_GROUP 8
#define FL_OFF 16

int flags;
char **mnt;
int mntcnt;

void parse_options(int argcnt, char **argstr)
{
	int ret;
	char cmdname[MAXFILENAMELEN], *slash = strrchr(argstr[0], '/');

	if (!slash)
		slash = argstr[0];
	else
		slash++;
	sstrncpy(cmdname, slash, MAXFILENAMELEN);

	while ((ret = getopt(argcnt, argstr, "Vfnugvah")) != EOF) {
		switch(ret) {
			case 'h':
			case '?':
usage:
				printf("Utility for turning quotas on and off.\nUsage: %s [-v] [-f | -n] [-u] [-g] (-a | mntpoint)\n", cmdname);
				printf("Bugs to %s\n", MY_EMAIL);
				exit(1);
			case 'V':
				version();
				exit(0);
			case 'u':
				flags |= FL_USER;
				break;
			case 'g':
				flags |= FL_GROUP;
				break;
			case 'v':
				flags |= FL_VERBOSE;
				break;
			case 'a':
				flags |= FL_ALL;
				break;
			case 'f':
				flags |= FL_OFF;
				break;
			case 'n':
				flags &= ~FL_OFF;
				break;
		}
	}
	if (optind + !(flags & FL_ALL) > argcnt) {
		fputs("Bad number of arguments.\n", stderr);
		goto usage;
	}
	if (!(flags & (FL_USER | FL_GROUP)))
		flags |= FL_USER | FL_GROUP;
	if (!(flags & FL_ALL)) {
		mnt = argstr + optind;
		mntcnt = argcnt - optind;
	}
}

/* Do the real operation */
void turn_it(char *mntpoint, char *dev, int type)
{
	int kernqt = user2kernquota(type);

	if (flags & FL_VERBOSE)
		printf("Turning %s quotas %s on device %s (%s)...", type2name(type), (flags & FL_OFF) ? "off" : "on",  dev, mntpoint);
	if (flags & FL_OFF) {
		if (quotactl(QCMD(Q_QUOTAOFF, kernqt), dev, 0, NULL) < 0)
			goto failed;
	}
	else {
		char file[MAXFILENAMELEN];
		get_qfname(mntpoint, type, file);
		if (quotactl(QCMD(Q_QUOTAON, kernqt), dev, 0, file) < 0) {
failed:
			printf("failed: %s\n", strerror(errno));
			if (errno == ENOENT || errno == EINVAL)
				puts("You haven't probably converted old quota files to new ones. See convertquota(8) manpage.");
			return;
		}
	}
	
	if (flags & FL_VERBOSE)
		puts("done");
}

void quotaonoff(int type)
{
	char devname[MAXDEVLEN];
	char mntpoint[MAXFILENAMELEN];

	init_mount(mnt, mntcnt);
	while (next_mount(type, devname, mntpoint, NULL))
		turn_it(mntpoint, devname, type);
	free_mount();
}

int main(int argcnt, char **argstr)
{
	parse_options(argcnt, argstr);
	
	if (flags & FL_USER)
		quotaonoff(QS_USRQUOTA);
	if (flags & FL_GROUP)
		quotaonoff(QS_GRPQUOTA);
	return 0;
}

